class FileSelector
  require 'tty-prompt'
  require 'set'

  def self.explore
    new.explore
  end

  def initialize
    @files = Set[]
    @prompt = TTY::Prompt.new
  end

  def explore
    explore_mode
    @files
  end

  def explore_mode(path = Dir.pwd)
    system 'clear'
    files = files_in path
    options = ['finish', 'include', File.dirname(path)] + files
    prompt = 'Files and directoies to explore',
    choice = @prompt.select(prompt, options, per_page: 20)
    if choice == 'include'
      include_mode(path)
      explore_mode(path)
    elsif choice != 'finish'
      if File.directory?(choice)
        explore_mode(choice) if File.directory?(choice)
      else
        system "less #{choice}" unless File.directory?(choice)
        explore_mode(path)
      end
    end
  end

  private

  def include_mode(dir)
    system 'clear'
    cwd_files = files_in dir
    prompt = 'Files and directoies to include'
    choices = @prompt.multi_select(prompt, [dir] + cwd_files, per_page: 20)
    @files.merge(paths_to_files(choices))
  end

  def paths_to_files(paths)
    # Converts a set of paths to non-directory files and directories to a set
    # of non-directory files by replacing directories with the files therein
    files = Set[]
    paths.each do |path|
      if !File.directory?(path)
        files.add(path)
      else
        files_to_add = Dir.glob("#{path}/**/*").reject do |f|
          File.directory?(f) || ['.', '..'].include?(f)
        end
        files.merge(files_to_add)
      end
    end
    files
  end

  def files_in(dir)
    (Dir[dir + '/.*'] + Dir[dir + '/*']) - ["#{dir}/.", "#{dir}/.."]
  end
end
